"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel' }"

"{ NameSpace: Smalltalk }"

MCVersionReader subclass:#TonelReader
	instanceVariableNames:'packageDirectory'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Core'
!

!TonelReader class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!TonelReader class methodsFor:'instance creation'!

on: aStringOrStream
        ^ self new stream: aStringOrStream

    "Created: / 09-07-2020 / 23:19:53 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelReader class methodsFor:'accessing'!

definitionOrders
        "Used to sort definitions inside a snapshot"
        ^ Dictionary withKeysAndValues: { 
                MCOrganizationDefinition.       0.
                MCMethodDefinition.                     1.
                MCClassDefinition.                      2.
                MCTraitDefinition.                      2.
                MCClassTraitDefinition.                 3 }

    "Modified: / 04-06-2019 / 12:07:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

extension
	"This is needed because the implementation of MCCacheRepository assumes each package 
	 has an extension (which is a lame implementation) and then we need to provide one. 
	 Since tonel is not cached (because we have our oen cache in the form of a local repository), 
	 we put something that will not be found anyway"
	^ 'NOTUSED'
		
! !

!TonelReader class methodsFor:'reading'!

on: parentDirectoryReference fileName: packageName
	"I will anster a reader for the package stored in 
	
		[repository]/parentDirectoryReference/packageName
	
	This operation will NOT read the package, but resulting instance will be prepared
	to doit, executing: #loadDefinitions"

	^ (self on: parentDirectoryReference)
		packageDirectory: packageName;
		yourself
! !

!TonelReader class methodsFor:'testing'!

canReadFileNamed: fileName
	"We validate that fileName does not contains dot, but that's the only thing we can do 
	 at this point."
	^ fileName noneSatisfy: [ :each | each = $. ]
!

isAbstract
	^ false
! !

!TonelReader methodsFor:'accessing'!

fileUtils
	^ TonelFileUtils current
!

packageDirectory
	^ packageDirectory
!

packageDirectory: aDirectoryName
        packageDirectory := self fileUtils 
                directoryFromPath: aDirectoryName 
                relativeTo: stream

    "Modified: / 09-07-2020 / 23:20:14 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelReader methodsFor:'loading'!

loadDefinitions
        | definitionOrders newDefinitions newCategories organization |
        
        definitionOrders := self class definitionOrders.
        newDefinitions := ((self packageDirectory directoryContentsAsFilenames
                select: [ :each | self canBeLoaded: each ]) 
                collect: [ :each | each readingFileDo: [ :s | TonelParser parseStream: s ] ])
                flatten
                sort: [ :a :b | (definitionOrders at: a class) < (definitionOrders at: b class) ].

        newCategories := self categoriesFrom: newDefinitions.
        (newCategories includes: self package name asSymbol)
                ifFalse: [ newCategories := newCategories copy addFirst: self package name; yourself ].
        organization := MCOrganizationDefinition categories: newCategories.
        
        definitions := (newDefinitions copyWithFirst: organization) sort

    "Modified: / 09-07-2020 / 23:21:56 / Jan Vrany <jan.vrany@labware.com>"
!

loadDependencies
	"There should not be dependencies"
	dependencies := #()
!

loadPackage
	package := MCPackage named: self packageNameFromPackageDirectory
!

loadVersionInfo
	info := MCVersionInfo
		name: self packageNameFromPackageDirectory, '-tonel.1'
		id: UUID new
		message: 'fabricated from a Tonel format repository'
		date: Date today
		time: Time now
		author: ''
		ancestors: #()
		stepChildren: #()
! !

!TonelReader methodsFor:'private'!

canBeLoaded: aFileReference
        | fileName |
        aFileReference isHidden ifTrue: [ ^ false ].
        fileName := self fileUtils fileName: aFileReference. 
        ^ fileName ~= 'package.st' and: [ fileName endsWith: '.st' ]

    "Modified: / 09-07-2020 / 21:15:16 / Jan Vrany <jan.vrany@labware.com>"
!

categoriesFrom: aCollection
        ^ ((aCollection select: #isClassDefinition)
                collect: #category
                as: OrderedSet)
                sorted: [ :a :b | a < b ]

    "Modified: / 07-07-2020 / 22:45:32 / Jan Vrany <jan.vrany@labware.com>"
!

packageNameFromPackageDirectory
    ^ packageDirectory baseName

    "Modified: / 07-07-2020 / 22:45:52 / Jan Vrany <jan.vrany@labware.com>"
! !

