"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel' }"

"{ NameSpace: Smalltalk }"

Object subclass:#TonelScanner
	instanceVariableNames:'stream'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Core'
!

TonelScanner comment:'I''m a generic scanner to help on parsing tonel files.'
!

!TonelScanner class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare

"
! !

!TonelScanner class methodsFor:'instance creation'!

new
    self error: 'Use #on:'
!

on: aStream
    ^ self basicNew 
        initializeStream: aStream;
        yourself
! !

!TonelScanner methodsFor:'initialization'!

initializeStream: aStream
    self initialize.
    stream := aStream
! !

!TonelScanner methodsFor:'private'!

newLine
    OperatingSystem isMSWINDOWSlike ifTrue: [
        ^ Character return , Character linefeed
    ].
    OperatingSystem isOSXlike ifTrue: [
        ^ Character return asString
    ].
    ^ Character linefeed asString

    "Modified: / 07-07-2020 / 11:49:43 / Jan Vrany <jan.vrany@labware.com>"
!

removeFrom: aString enclosingStart: startChar end: endChar
    ^ self 
        removeFrom: aString 
        enclosingStart: startChar 
        end: endChar
        clean: #both
!

removeFrom: aString enclosingStart: startChar end: endChar clean: cleanSymbol
    "cleanSymbol can be #left, #right and #both" 
    | nl result stop ch start end |
    
    nl := self newLine.
    result := (aString withoutSeparators withLineEndings: nl) readStream.
    (result peek = startChar) 
        ifFalse: [ TonelParseError signal: 'I cannot remove enclosing start' ].
    result skip: 1.
    
    (#(both left) includes: cleanSymbol) 
        ifTrue: [ 
            stop := nl size.
            [ (stop > 0) and: [ self isSeparator: (ch := result peek) ] ]
            whileTrue: [ 
                (self isEnter: ch) ifTrue: [ stop := stop - 1 ].
                result skip: 1 ] ].
    start := result position.
    
    result setToEnd.
    result skip: -1.
    (result peek = endChar) 
        ifFalse: [ TonelParseError signal: 'I cannot remove enclosing end' ].
    result skip: -1.
    (#(both right) includes: cleanSymbol)
        ifTrue: [ 
            stop := nl size.
            [ (stop > 0) and: [ self isSeparator: (ch := result peek) ] ]
            whileTrue: [ 
                (self isEnter: ch) ifTrue: [ stop := stop - 1 ].
                result skip: -1. ] ].
    end := result position.
    
    ^ result contents
        copyFrom: start + 1 
        to: end + 1

    "Modified: / 25-05-2020 / 14:33:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-07-2020 / 23:28:11 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelScanner methodsFor:'scanning'!

scan 
    self subclassResponsibility
! !

!TonelScanner methodsFor:'testing'!

isEnter: aCharacter
    ^ #(13 10) includes: aCharacter asciiValue
!

isSeparator: aCharacter
    ^ aCharacter isSeparator
! !

!TonelScanner class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

