"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel/tests' }"

"{ NameSpace: Smalltalk }"

MCTestCase subclass:#TonelParserTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Tests'
!

!TonelParserTest class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare

"
! !

!TonelParserTest methodsFor:'asserting'!

assertParse: aString rule: rule equals: result 
    self 
        assert: (self parse: aString rule: rule)
        equals: result
!

shouldParse: aString rule: rule raise: error 
    | parser |
    
    parser := TonelParser on: aString readStream.
    self 
        should: [ parser perform: rule ]
        raise: error
! !

!TonelParserTest methodsFor:'private'!

parse: aString rule: rule
    | parser |
    
    parser := TonelParser on: aString readStream.
     ^ parser perform: rule.
    
! !

!TonelParserTest methodsFor:'tests'!

testComment
    self 
        shouldParse: 'this "should" be an error'
        rule: #comment
        raise: TonelParseError.
    
    self  
        assertParse: '"this is a comment"'
        rule: #comment
        equals: 'this is a comment'.
    
    self  
        assertParse: '"""this"" is a comment with ""nested"" colons ""also at the end"""'
        rule: #comment
        equals: '"this" is a comment with "nested" colons "also at the end"'	
        
        
!

testExtractSelector
    | parser |

    parser := TonelParser new.

    self assert: (parser extractSelector: 'unary') equals: #unary.
    self assert: (parser extractSelector: '+ something') equals: #+.
    self assert: (parser extractSelector: '==> other') equals: #'==>' .
    self 
        assert: (parser extractSelector: 'some: arg1 keyword: arg2 selector: arg3') 
        equals: #some:keyword:selector:.
    self 
        assert: (parser extractSelector: 'some:         arg1 keyword:arg2 selector: arg3') 
        equals: #some:keyword:selector:.
    self 
        assert: (parser extractSelector: 'some: arg1 
keyword: arg2 
selector: arg3') 
        equals: #some:keyword:selector:.

    "Created: / 25-05-2020 / 13:15:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testExtractSelectorWithComments
    | parser |

    parser := TonelParser new.

    self assert: (parser extractSelector: 'unary "comment"') equals: #unary.
    self assert: (parser extractSelector: '+ "comment" something') equals: #+.
    self assert: (parser extractSelector: '==> "comment" other') equals: #'==>'.
    self 
        assert: (parser extractSelector: 'some: arg1 "comment"  keyword: arg2  "comment" selector: arg3 "comment"') 
        equals: #some:keyword:selector:.
    self 
        assert: (parser extractSelector: 'some:         arg1 "comment" keyword:arg2 selector: arg3') 
        equals: #some:keyword:selector:.
    self 
        assert: (parser extractSelector: 'some: arg1 "comment"
keyword: arg2 "comment"
selector: arg3') 
        equals: #some:keyword:selector:.

    "Created: / 25-05-2020 / 13:15:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

testLiteralDollarCharacterAtEndOfLiteralArray
    self 
        assertParse: '[^#($$)]'
        rule: #methodBody
        equals: '^#($$)'.
!

testMaybeReturnADollar
    self 
        assertParse: '[self testSomething ifTrue: [^$$]]'
        rule: #methodBody
        equals: 'self testSomething ifTrue: [^$$]'.
!

testMetadata
    self 
        assertParse: '{ #vars: [ #a, #b ] }' 
        rule: #metadata 
        equals: (Dictionary with: #vars -> #(a b)).
    
    self 
        assertParse: '{ 
    #vars: [ #a, #b ],
    #uses: { #someNested: 42 } 
    }' 
        rule: #metadata 
        equals: 
            (Dictionary 
                with: #vars -> #(a b)
                with: #uses -> (Dictionary with:#someNested -> 42)).

    "Modified: / 27-05-2020 / 15:58:31 / Jan Vrany <jan.vrany@labware.com>"
!

testMethod
    self 
        assertParse: 'Object>>name' 
        rule: #method
        equals: #(('Object' nil) 'name').
        
    self 
        assertParse: 'Object >> name: aString' 
        rule: #method
        equals: #(('Object' nil) 'name: aString').
        
    self 
        assertParse: 'Object >> name: aString [ I do not care ]' 
        rule: #method
        equals: #(('Object' nil) 'name: aString').
        
    self 
        assertParse: 'Object class >> name: aString' 
        rule: #method
        equals: #(('Object' 'class') 'name: aString').
!

testMethodBody
    self 
        assertParse: '[ method body... I will ignore what is inside ]'
        rule: #methodBody
        equals: ' method body... I will ignore what is inside'.

    self 
        assertParse: '[
method 
    [body... [I 
        will ignore] 
what] is inside
]'
        rule: #methodBody
        equals: '
method 
    [body... [I 
        will ignore] 
what] is inside'.
        
    self 
        assertParse: '[ method body with "''", ''"'', "[", '']'' ]'
        rule: #methodBody
        equals: ' method body with "''", ''"'', "[", '']'''.
    
!

testMethodBodyWithBlockContainingDollarCharacter
    "https://github.com/pharo-vcs/tonel/issues/44"

    self assertParse: '[ $$ ]' rule: #methodBody equals: ' $$'.
    self assertParse: '[$$ ]' rule: #methodBody equals: '$$'.
    self assertParse: '[ $$]' rule: #methodBody equals: ' $$'.
    self assertParse: '[$$]' rule: #methodBody equals: '$$'
!

testMethodBodyWithCommentNextToDollarCharLiteral
    self 
        assertParse: '[^$$"what about this one]"]'
        rule: #methodBody
        equals: '^$$"what about this one]"'
!

testMethodBodyWithSquareBracketInsideLiteralArray
    self 
        assertParse: '[ "What about this valid literal Array guys" ^#( [ )]'
        rule: #methodBody
        equals: ' "What about this valid literal Array guys" ^#( [ )'.
!

testMethodBodyWithSquareBracketInsideLiteralArray2
    self 
        assertParse: '[^#( "note that we need to prefix ) with $, but this is not necessary for ] and }" '')]}'' $) ] } )]'
        rule: #methodBody
        equals: '^#( "note that we need to prefix ) with $, but this is not necessary for ] and }" '')]}'' $) ] } )'.
!

testMethodBodyWithTrickyDollarCharLiteralInLiteralArray
    self 
        assertParse: '[^#( $$''please use space before this string)(''$$"please use space before this comment or avoid putting a ] in it"$$(this really is a literal array with a ] in it))]'
        rule: #methodBody
        equals: '^#( $$''please use space before this string)(''$$"please use space before this comment or avoid putting a ] in it"$$(this really is a literal array with a ] in it))'
!

testMethodDef
    self 
        assertParse: '
{ #category: ''accessing'' }
Object>>name [
    ^ self printString
]'
        rule: #methodDef
        equals: (MCMethodDefinition
            className: #Object
            classIsMeta: false
            selector: #name
            category: 'accessing' 
            timeStamp: nil
            source: 'name
    ^ self printString').
            
    self 
        assertParse: '
Object class>>name [ 
    ^ self printString
]'
        rule: #methodDef
        equals: (MCMethodDefinition
            className: #Object
            classIsMeta: true
            selector: #name
            category: ''
            timeStamp: nil
            source: 'name 
    ^ self printString').

    self 
        assertParse: '
TClass classSide >> template: aSystemCategoryName [ 
    "I really do not care"
]'
        rule: #methodDef
        equals: (MCMethodDefinition
            className: #TClass
            classIsMeta: true
            selector: #template:
            category: ''
            timeStamp: nil
            source: 'template: aSystemCategoryName 
    "I really do not care"').
!

testMethodDefList
    | parsed |
    
    parsed := self 
        parse: '
Object class>>new [
    ^ self basicNew initialize
]

{ #category: ''accessing'' }
Object>>name [
    ^ self printString
]

{ #category: ''printing'' }
Object>>printOn: aStream [
    "Append to the argument, aStream, a sequence of characters that  
    identifies the receiver."

    | title |
    title := self class name.
    aStream
        nextPutAll: (title first isVowel ifTrue: [''an ''] ifFalse: [''a '']);
        nextPutAll: title

]'
        rule: #methodDefList.
        
    self assert: parsed size equals: 3
!

testSplitMethodSourceIntoCorrupted
    | writer definition |

    writer := TonelWriter new.

    "Sometimes data loaded from old MCZs are corrupted and the selector doesn't match the source, resulting in infinite search loop."
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #name
        category: 'accessing'
        timeStamp: nil
        source:
            'named: aString
    ^ self named: aString environment: self defaultEnvironment'.
    self
        should: [ writer splitMethodSource: definition into: [ :d :s |  ] ]
        raise: TonelWriteError

    "Created: / 25-05-2020 / 13:15:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 09-07-2020 / 20:33:06 / Jan Vrany <jan.vrany@labware.com>"
!

testType
    self assertParse: 'Class' rule: #type equals: 'Class'.
    self assertParse: 'Trait' rule: #type equals: 'Trait'.
    self assertParse: 'Extension' rule: #type equals: 'Extension'
!

testTypeDef
    self 
        assertParse: '
"
this is a test
"
Class { 
    #name: ''X'',
    #superclass: ''Y'',
    #category: ''Z'' 
}'
        rule: #typeDef
        equals: (MCClassDefinition
        name: 'X'
        superclassName: 'Y'
        traitComposition: '{}'
        classTraitComposition: '{}'
        category: 'Z'
        instVarNames: #()
        classVarNames: #()
        poolDictionaryNames: #()
        classInstVarNames: #()
        type: #normal
        comment: 'this is a test'
        commentStamp: nil)
! !

!TonelParserTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

