"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel/tests' }"

"{ NameSpace: Smalltalk }"

MCTestCase subclass:#TonelWriterTest
	instanceVariableNames:'directory'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Tests'
!

!TonelWriterTest class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!TonelWriterTest methodsFor:'accessing'!

actualClass
    ^ TonelWriter
! !

!TonelWriterTest methodsFor:'private'!

expectedMCMockASubclassClassSt
    ^ 'Class {
    #name : #MCMockASubclass,
    #superclass : #MCMockClassA,
    #instVars : [
        ''x''
    ],
    #classVars : [
        ''Y''
    ],
    #category : #MonticelloMocks
}

{ #category : #''as yet unclassified'' }
MCMockASubclass class >> initialize [

    InitializationOrder := InitializationOrder
        ifNil: [ -100 ] "let the test fail"
        ifNotNil: [ InitializationOrder + 1.]
]

{ #category : #''as yet unclassified'' }
MCMockASubclass >> variables [
    ^ x + Y + MCMockClassA
]

{ #category : #''as yet unclassified'' }
MCMockASubclass >> variables2 [
    ^ ivar + CVar
]
'
!

expectedMCMockClassAClassSt
    ^ '"
This is a mock class. The Monticello tests manipulated it to simulate a developer modifying code in the image.
"
Class {
        #name : #MCMockClassA,
        #superclass : #MCMock,
        #instVars : [
            ''ivar''
        ],
        #classVars : [
            ''CVar'',
            ''InitializationOrder''
        ],
        #category : #MonticelloMocks
}

{ #category : #''as yet unclassified'' }
MCMockClassA class >> cVar [
    ^ CVar
]

{ #category : #''as yet unclassified'' }
MCMockClassA class >> initializationOrder [

    ^ InitializationOrder
]

{ #category : #''as yet unclassified'' }
MCMockClassA class >> initialize [
    CVar := #initialized.
    InitializationOrder := 1.

]

{ #category : #''as yet unclassified'' }
MCMockClassA class >> one [

    ^ 1
]

{ #category : #''as yet unclassified'' }
MCMockClassA class >> touchCVar [
    CVar := #touched
]

{ #category : #numeric }
MCMockClassA >> a [
    ^ ''a2''
]

{ #category : #numeric }
MCMockClassA >> b [
    ^ ''b1''
]

{ #category : #numeric }
MCMockClassA >> c [
    ^ ''c1''
]

{ #category : #numeric }
MCMockClassA >> d [
    ^ ''d''
]

{ #category : #boolean }
MCMockClassA >> falsehood [
    ^ false
]

{ #category : #boolean }
MCMockClassA >> moreTruth [

    ^ true
]

{ #category : #numeric }
MCMockClassA >> one [
    ^ 1
]

{ #category : #''drag\''n\''drop'' }
MCMockClassA >> q [
]

{ #category : #boolean }
MCMockClassA >> truth [
    ^ true
]

{ #category : #numeric }
MCMockClassA >> two [
    ^ 2
]
'

    "Modified: / 09-07-2020 / 21:27:39 / Jan Vrany <jan.vrany@labware.com>"
!

expectedMCSnapshotTestExtensionSt
    ^ 'Extension { #name : #MCSnapshotTest }

{ #category : #''*MonticelloMocks'' }
MCSnapshotTest >> mockClassExtension [

    "I change the protocol of this method to resolve the failing test: MCChangeNotificationTest >> testExtMethodModified. This test basically test that when we modified an extension method, the extended package is marked as ''modified''. The problem is that Monticello treat differently a classic method from an extension method, and this only by checking if the protocol name start with a star. Therefore, if the protocol does not match the extending package name, the extending package name will never be notified, and the test will fail. " 
]
'
!

nativeLineEnding
    OperatingSystem isMSWINDOWSlike ifTrue: [
        ^ Character return , Character linefeed
    ].
    OperatingSystem isOSXlike ifTrue: [
        ^ Character return asString
    ].
    ^ Character linefeed asString

    "Created: / 07-07-2020 / 11:37:15 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelWriterTest methodsFor:'running'!

setUp
    super setUp.
    directory := Filename newTemporaryDirectory.

    "Created: / 07-07-2020 / 21:04:19 / Jan Vrany <jan.vrany@labware.com>"
!

tearDown
    directory exists ifTrue: [
        directory recursiveRemove.
    ].
    super tearDown.

    "Created: / 07-07-2020 / 21:04:49 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelWriterTest methodsFor:'tests'!

checkFilesStructureIn: aFileReference

    | nl |
    
    self assert: (aFileReference / 'Monticello-Mocks') exists.
    self 
        assert: ((aFileReference / 'Monticello-Mocks') directoryContents asArray sort) 
        equals: #(
            'MCMockASubclass.class.st' 
            'MCMockClassA.class.st' 
            'MCMockClassB.class.st' 
            'MCMockClassD.class.st' 
            'MCMockClassE.class.st' 
            'MCMockClassF.class.st' 
            'MCMockClassG.class.st' 
            'MCMockClassH.class.st' 
            'MCMockClassI.class.st' 
            'MCSnapshotTest.extension.st'
            'package.st').
    
    nl := self nativeLineEnding.
    self 
        assert: (aFileReference / 'Monticello-Mocks' / 'MCMockClassA.class.st') contentsAsString
        equals: (self expectedMCMockClassAClassSt withLineEndings: nl).
    self 
        assert: (aFileReference / 'Monticello-Mocks' / 'MCMockASubclass.class.st') contentsAsString
        equals: (self expectedMCMockASubclassClassSt withLineEndings: nl).
    self 
        assert: (aFileReference / 'Monticello-Mocks' / 'MCSnapshotTest.extension.st') contentsAsString
        equals: (self expectedMCSnapshotTestExtensionSt withLineEndings: nl)

    "Modified: / 09-07-2020 / 21:27:06 / Jan Vrany <jan.vrany@labware.com>"
!

testSourceCodeOf
    TonelMock comment: 'Description
--------------------

I am a simple Mock for Tonel export test
'.
    self
        assert: (self actualClass sourceCodeOf: TonelMock)
        equals:
            ('"
Description
--------------------

I am a simple Mock for Tonel export test
"
Class {
        #name : #TonelMock,
        #superclass : #Object,
        #category : #''SCM-Monticello-Tonel-Tests''
}

{ #category : #accessing }
TonelMock class >> classMethod [
    ^ 8
]

{ #category : #accessing }
TonelMock >> instanceMethod [
    ^ true
]
'
                withLineEndings: self nativeLineEnding)

    "Modified: / 07-07-2020 / 21:33:28 / Jan Vrany <jan.vrany@labware.com>"
!

testSplitMethodSourceInto
    | writer declaration source definition newLine tab space |
    
    newLine := self nativeLineEnding.
    tab := Character tab asString.
    space := Character space asString.
    writer := self actualClass new.

    "simplest split"
    definition := MCMethodDefinition
        className: #SomeClass 
        selector: #name 
        category: 'accessing' 
        timeStamp: nil
        source: 'name
    ^ self'.            
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    
    self assert: declaration equals: 'name'.
    self assert: source equals: newLine, tab, '^ self'.
    
    "test space at the end of method declaration (it needs to be kept)"
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #name:
        category: 'accessing'
        timeStamp: nil
        source: 'name: aString 
    name := aString'.
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    self assert: declaration equals: 'name: aString'.
    self assert: source equals: space, newLine, tab, 'name := aString'.

    "test multiline declaration"
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #method:with:several:lines:
        category: 'accessing'
        timeStamp: nil
        source: 'method: var1
    with: var2
    several: var3
    lines: var4
    
    ^ var1 + var2 + var3 + var4'.
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    self assert: declaration equals: ('method: var1
    with: var2
    several: var3
    lines: var4' withLineEndings: self nativeLineEnding).
    self assert: source equals: (newLine, tab, newLine, tab, '^ var1 + var2 + var3 + var4' withLineEndings: newLine).

    "test comment before declaration (it may happen, if someone copied from diffmorph)"
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #name:
        category: 'accessing'
        timeStamp: nil
        source: '
"protocol: accessing"
name: aString 
    name := aString'.
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    self assert: declaration equals: 'name: aString'.
    self assert: source equals: space, newLine, tab, 'name := aString'.

    "test source right after declaration (no enter between selector and source)"
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #name:
        category: 'accessing'
        timeStamp: nil
        source: 'name: aString name := aString'.
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    self assert: declaration equals: 'name: aString'.
    self assert: source equals: ' name := aString'.
    
    "test method name containin name of keywords in variables"
    definition := MCMethodDefinition
        className: #SomeClass
        selector: #a:b:c:
        category: 'accessing'
        timeStamp: nil
        source: 'a: b b: c c: d ^ 42'.
    writer splitMethodSource: definition into: [ :d :s | declaration := d. source := s ].
    self assert: declaration equals: 'a: b b: c c: d'.
    self assert: source equals: ' ^ 42'.

    "Modified: / 07-07-2020 / 11:38:31 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteBinaryInstanceMethodDefinitionWithDotNextToArgument
    | writer def stream |
    
    writer := self actualClass new.

    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: false
        selector: #+
        category: 'accessing' 
        timeStamp: nil
        source: '+ a.
    ^ 42'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #accessing }
Object >> + a [.
    ^ 42
]
' withLineEndings: self nativeLineEnding).

    "Modified: / 07-07-2020 / 11:38:37 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteClassDefinitionOn
    | writer def stream |
    
    writer := self actualClass new.

    stream := String new writeStream.
    def := MCClassDefinition
        name: #SomeObject
        superclassName: #Object
        traitComposition: '{}'
        classTraitComposition: '{}'
        category: #Kernel
        instVarNames: #()
        classVarNames: #()
        poolDictionaryNames: #()
        classInstVarNames: #()
        type: #normal
        comment: 'comment test'
        commentStamp: nil.
    writer writeClassDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('"
comment test
"
Class {
        #name : #SomeObject,
        #superclass : #Object,
        #category : #Kernel
}
' withLineEndings: self nativeLineEnding).

    stream := String new writeStream.
    def := MCClassDefinition
        name: #SomeObject
        superclassName: #Object
        traitComposition: '{}'
        classTraitComposition: '{}'
        category: #Kernel
        instVarNames: #(a b c)
        classVarNames: #(D E)
        poolDictionaryNames: #(POOL)
        classInstVarNames: #(instVarA)
        type: #normal
        comment: 'comment test'
        commentStamp: nil.
    writer writeClassDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('"
comment test
"
Class {
    #name : #SomeObject,
    #superclass : #Object,
    #instVars : [
        ''a'',
        ''b'',
        ''c''
    ],
    #classVars : [
        ''D'',
        ''E''
    ],
    #pools : [
        ''POOL''
    ],
    #classInstVars : [
        ''instVarA''
    ],
    #category : #Kernel
}
' withLineEndings: self nativeLineEnding).

    "Modified: / 09-07-2020 / 21:16:51 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteKeywordInstanceMethodDefinitionWithDotNextToArgument
    | writer def stream |
    
    writer := self actualClass new.

    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: false
        selector: #min:max
        category: 'accessing' 
        timeStamp: nil
        source: 'min: a max: b.
    ^ 42'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #accessing }
Object >> min: a max: b [.
    ^ 42
]
' withLineEndings: self nativeLineEnding).

    "Modified: / 07-07-2020 / 11:38:51 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteMethodDefinitionOn
    | writer def stream |
    
    writer := self actualClass new.

    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: false
        selector: #selector
        category: 'accessing' 
        timeStamp: nil
        source: 'selector
    ^ 42'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #accessing }
Object >> selector [
    ^ 42
]
' withLineEndings: self nativeLineEnding).

    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: true
        selector: #selector
        category: 'accessing' 
        timeStamp: nil
        source: 'selector
    ^ 42'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #accessing }
Object class >> selector [
    ^ 42
]
' withLineEndings: self nativeLineEnding).


    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: false
        selector: #=
        category: 'comparing' 
        timeStamp: nil
        source: '= anObject
    ^ self == anObject'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #comparing }
Object >> = anObject [
    ^ self == anObject
]
' withLineEndings: self nativeLineEnding).

    "Modified: / 07-07-2020 / 11:39:03 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteSnapshot
    | writer |

    writer := self actualClass on: directory.
    writer writeSnapshot: self mockSnapshot.
    
    self checkFilesStructureIn: directory

    "Modified: / 07-07-2020 / 21:05:12 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteSnapshotWithoutOrganization
    "test if the exported code has correct structure even if the Monticello snapshot does not have
    a proper organization definition so the structure must be taken from the category names. 
    See TonelWriter>>#createDefaultOrganizationFrom:"

    | aSnapshot writer |
    
    aSnapshot := self mockSnapshot copy.
    (aSnapshot definitions select: #isOrganizationDefinition) do: [ :each | 
        aSnapshot definitions remove: each ].

    writer := self actualClass on: directory.
    writer writeSnapshot: self mockSnapshot.
    
    self checkFilesStructureIn: directory

    "Modified: / 09-07-2020 / 21:23:20 / Jan Vrany <jan.vrany@labware.com>"
!

testWriteUnaryInstanceMethodDefinitionWithDotNextToSelector
    | writer def stream |
    
    writer := self actualClass new.

    stream := String new writeStream.
    def := MCMethodDefinition 
        className: #Object
        classIsMeta: false
        selector: #selector
        category: 'accessing' 
        timeStamp: nil
        source: 'selector.
    ^ 42'.
    writer writeMethodDefinition: def on: stream. 
    self 
        assert: stream contents 
        equals: ('
{ #category : #accessing }
Object >> selector [.
    ^ 42
]
' withLineEndings: self nativeLineEnding).

    "Modified: / 07-07-2020 / 11:39:10 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelWriterTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

